@@ -10,8 +10,13 @@ from audeer.core.io import (
     list_dir_names,
     list_file_names,
     mkdir,
+    move_file,
     replace_file_extension,
     rmdir,
+    touch,
+)
+from audeer.core.path import (
+    path,
     safe_path,
 )
 from audeer.core.tqdm import (
@@ -26,6 +31,7 @@ from audeer.core.utils import (
     freeze_requirements,
     git_repo_tags,
     git_repo_version,
+    install_package,
     is_semantic_version,
     is_uid,
     run_tasks,
@@ -34,6 +40,10 @@ from audeer.core.utils import (
     to_list,
     uid,
 )
+from audeer.core.version import (
+    LooseVersion,
+    StrictVersion,
+)
 
 
 # Discourage from audeer import *
