@@ -2,6 +2,7 @@ import os
 import platform
 import stat
 import tarfile
+import time
 import zipfile
 
 import pytest
@@ -158,6 +159,8 @@ def test_common_directory(dirs, expected):
     _, expected = os.path.splitdrive(expected)
     common = common.replace('\\', '/')
     expected = expected.replace('\\', '/')
+    # On MacOS we get a '/System/Volumes/Data' in front
+    common = common.replace('/System/Volumes/Data', '')
     assert common == expected
 
 
@@ -188,12 +191,19 @@ def test_file_extension(path, extension):
     assert type(ext) is str
 
 
-@pytest.mark.parametrize('dir_list', [
-    [],
-    ['a', 'b', 'c'],
-    ['a'],
-])
-def test_list_dir_names(tmpdir, dir_list):
+@pytest.mark.parametrize(
+    'dir_list,recursive',
+    [
+        ([], False),
+        ([], True),
+        (['a', 'b', 'c'], False),
+        (['a', 'b', 'c'], True),
+        (['a'], False),
+        (['a'], True),
+        (['a', os.path.join('a', 'b'), os.path.join('a', 'b', 'c')], True),
+    ],
+)
+def test_list_dir_names(tmpdir, dir_list, recursive):
     dir_tmp = tmpdir.mkdir('folder')
     directories = []
     for directory in dir_list:
@@ -204,35 +214,148 @@ def test_list_dir_names(tmpdir, dir_list):
         assert os.path.isdir(directory)
 
     path = os.path.join(str(dir_tmp), '.')
-    dirs = audeer.list_dir_names(path)
+    dirs = audeer.list_dir_names(
+        path,
+        recursive=recursive,
+    )
     assert dirs == sorted(directories)
     assert type(dirs) is list
 
+    # test basenames
+    dirs = audeer.list_dir_names(
+        path,
+        recursive=recursive,
+        basenames=True,
+    )
+    assert dirs == sorted(dir_list)
 
-@pytest.mark.parametrize('files,path,filetype,file_list', [
-    ([], '.', '', []),
-    ([], '.', 'wav', []),
-    (['t1.wav', 't2.wav', 't3.ogg'], '.', '', ['t1.wav', 't2.wav', 't3.ogg']),
-    (['t1.wav', 't2.wav', 't3.ogg'], '.', 'ogg', ['t3.ogg']),
-    (['t1.wav'], 't1.wav', '', ['t1.wav']),
-])
-def test_list_file_names(tmpdir, files, path, filetype, file_list):
+
+@pytest.mark.parametrize(
+    'files,path,filetype,file_list,recursive',
+    [
+        ([], '.', '', [], False),
+        ([], '.', '', [], True),
+        ([], '.', 'wav', [], False),
+        ([], '.', 'wav', [], True),
+        (
+            ['t3.ogg', 't2.wav', 't1.wav'],
+            '.',
+            '',
+            ['t1.wav', 't2.wav', 't3.ogg'],
+            False,
+        ),
+        (
+            [
+                't3.ogg',
+                't2.wav',
+                't1.wav',
+                os.path.join('sub', 't1.wav'),
+                os.path.join('sub', 't1.ogg'),
+            ],
+            '.',
+            '',
+            ['t1.wav', 't2.wav', 't3.ogg'],
+            False,
+        ),
+        (
+            [
+                't3.ogg',
+                't2.wav',
+                't1.wav',
+                os.path.join('sub', 't1.wav'),
+                os.path.join('sub', 'sub', 't1.ogg'),
+            ],
+            '.',
+            '',
+            [
+                os.path.join('sub', 'sub', 't1.ogg'),
+                os.path.join('sub', 't1.wav'),
+                't1.wav',
+                't2.wav',
+                't3.ogg',
+            ],
+            True,
+        ),
+        (
+            [
+                't3.ogg',
+                't2.wav',
+                't1.wav',
+                os.path.join('sub', 't1.wav'),
+                os.path.join('sub', 'sub', 't1.ogg'),
+            ],
+            '.',
+            'ogg',
+            ['t3.ogg'],
+            False,
+        ),
+        (
+            [
+                't3.ogg',
+                't2.wav',
+                't1.wav',
+                os.path.join('sub', 't1.wav'),
+                os.path.join('sub', 'sub', 't1.ogg'),
+            ],
+            '.',
+            'ogg',
+            [
+                os.path.join('sub', 'sub', 't1.ogg'),
+                't3.ogg',
+            ],
+            True,
+        ),
+        (
+            [
+                't1.wav',
+                os.path.join('sub', 't1.wav'),
+            ],
+            't1.wav',
+            '',
+            ['t1.wav'],
+            False,
+        ),
+        (
+            [
+                't1.wav',
+                os.path.join('sub', 't1.wav'),
+            ],
+            't1.wav',
+            '',
+            ['t1.wav'],
+            True,
+        ),
+    ],
+)
+def test_list_file_names(tmpdir, files, path, filetype, file_list, recursive):
+    file_list_org = file_list.copy()
     dir_tmp = tmpdir.mkdir('folder')
     dir_tmp.mkdir('subfolder')
     path = os.path.join(str(dir_tmp), path)
     for file in files:
         # Create the files
         file_tmp = dir_tmp.join(file)
+        audeer.mkdir(os.path.dirname(file_tmp))
         file_tmp.write('')
     if os.path.isdir(path):
-        file_list = [
-            audeer.safe_path(os.path.join(path, f)) for f in file_list
-        ]
+        file_list = [audeer.path(path, f) for f in file_list]
     else:
         file_list = [path]
-    f = audeer.list_file_names(path, filetype=filetype)
+    f = audeer.list_file_names(
+        path,
+        filetype=filetype,
+        recursive=recursive,
+    )
     assert f == file_list
     assert type(f) is list
+    # test basenames
+    f = audeer.list_file_names(
+        path,
+        filetype=filetype,
+        recursive=recursive,
+        basenames=True,
+    )
+    assert f == file_list_org
 
 
 def test_mkdir(tmpdir):
@@ -308,6 +431,34 @@ def test_mkdir(tmpdir):
     assert mode != int('775', 8)
 
 
+@pytest.mark.parametrize(
+    'src_file, dst_file',
+    [
+        (
+            'file1',
+            'file1',
+        ),
+        (
+            'file1',
+            'file2',
+        ),
+    ]
+)
+def test_move_file(tmpdir, src_file, dst_file):
+
+    tmp_path = str(tmpdir.mkdir('folder'))
+    tmp_path = audeer.mkdir(tmp_path)
+
+    src_path = audeer.touch(os.path.join(tmp_path, src_file))
+    dst_path = os.path.join(tmp_path, dst_file)
+
+    audeer.move_file(src_path, dst_path)
+
+    if src_file != dst_file:
+        assert not os.path.exists(src_path)
+    assert os.path.exists(dst_path)
+
+
 def test_rmdir(tmpdir):
     # Non existing dir
     audeer.rmdir('non-esitent')
@@ -337,46 +488,6 @@ def test_rmdir(tmpdir):
     os.chdir(current_path)
 
 
-@pytest.mark.parametrize('path', [
-    ('~/.someconfigrc'),
-    ('file.tar.gz'),
-    ('/a/c.d/g'),
-    ('/a/c.d/g.exe'),
-    ('../.././README.md'),
-    ('folder/file.txt'),
-    (b'folder/file.txt'),
-    (''),
-])
-def test_safe_path(path):
-    if path:
-        expected_path = os.path.abspath(os.path.expanduser(path))
-    else:
-        expected_path = ''
-    if type(expected_path) == bytes:
-        expected_path = expected_path.decode('utf8')
-    path = audeer.safe_path(path)
-    assert path == expected_path
-    assert type(path) is str
-
-
-def test_safe_path_symlinks(tmpdir):
-    filename = 'file.txt'
-    linkname = 'link.txt'
-    dir_tmp = tmpdir.mkdir('folder')
-    f = dir_tmp.join(filename)
-    f.write('')
-    folder = audeer.mkdir(str(dir_tmp))
-    file = os.path.join(folder, filename)
-    link = os.path.join(folder, linkname)
-    os.symlink(file, link)
-    expected_path = os.path.realpath(os.path.expanduser(link))
-    path = audeer.safe_path(link)
-    _, path = os.path.splitdrive(path)
-    _, expected_path = os.path.splitdrive(expected_path)
-    assert path == expected_path
-    assert type(path) is str
-
-
 @pytest.mark.parametrize(
     'path, new_extension, expected_path',
     [
@@ -388,3 +499,19 @@ def test_safe_path_symlinks(tmpdir):
 def test_replace_file_extension(path, new_extension, expected_path):
     path = audeer.replace_file_extension(path, new_extension)
     assert path == expected_path
+
+
+def test_touch(tmpdir):
+    path = str(tmpdir.mkdir('folder1'))
+    path = audeer.mkdir(path)
+    path = os.path.join(path, 'file')
+    assert not os.path.exists(path)
+    audeer.touch(path)
+    assert os.path.exists(path)
+    stat = os.stat(path)
+    time.sleep(.1)
+    audeer.touch(path)
+    assert os.path.exists(path)
+    new_stat = os.stat(path)
+    assert stat.st_atime < new_stat.st_atime
+    assert stat.st_mtime < new_stat.st_mtime
