@@ -59,6 +59,35 @@ def test_deprecated():
         assert expected_message == str(w[-1].message)
 
 
+def test_deprecated_default_value():
+
+    @audeer.deprecated_default_value(
+        argument='foo',
+        new_default_value='bar',
+        change_in_version='1.0.0',
+    )
+    def function_with_deprecated_default_value(*, foo='foo'):
+        return foo
+
+    expected_message = (
+        "The default of 'foo' will change from "
+        "'foo' to 'bar' "
+        "with version 1.0.0."
+    )
+    default_value = 'foo'
+    with pytest.warns(None):
+        # no warning if we set value
+        function_with_deprecated_default_value(foo='foo')
+        function_with_deprecated_default_value(foo='bar')
+    with warnings.catch_warnings(record=True) as w:
+        # Cause all warnings to always be triggered.
+        warnings.simplefilter("always")
+        # Raise warning
+        assert function_with_deprecated_default_value() == default_value
+        assert issubclass(w[-1].category, UserWarning)
+        assert expected_message == str(w[-1].message)
+
+
 def test_deprecated_keyword_argument():
 
     @audeer.deprecated_keyword_argument(
@@ -189,6 +218,71 @@ def test_git_repo_version():
     assert version == expected_version[1:]
 
 
+@pytest.mark.parametrize(
+    'version, is_semantic',
+    [
+        (
+            '1.0.0',
+            True,
+        ),
+        (
+            'v1.0.0',
+            True,
+        ),
+        (
+            '4.0.0-20200206.095534-3',
+            True,
+        ),
+        (
+            'v1.0.1-1-gdf29c4a',
+            True,
+        ),
+        (
+            '1',
+            False,
+        ),
+        (
+            'v1',
+            False,
+        ),
+        (
+            'v1.3-r3',
+            False,
+        ),
+        (
+            'v1.3.3-r3',
+            True,
+        ),
+        (
+            'a.b.c',
+            False,
+        ),
+        (
+            'va.b.c',
+            False,
+        ),
+        (
+            '1.2.a',
+            False,
+        ),
+        (
+            'v1.3.3.3-r3',
+            False,
+        ),
+        (
+            '1.0.0+20130313144700',
+            True,
+        ),
+        (
+            '1.0.0-alpha+001',
+            True,
+        ),
+    ]
+)
+def test_is_semantic_version(version, is_semantic):
+    assert audeer.is_semantic_version(version) == is_semantic
+
+
 @pytest.mark.parametrize(
     'uid, expected',
     [
@@ -257,10 +351,89 @@ def test_run_tasks(multiprocessing, num_workers, task_fun, params):
 def test_run_worker_threads(func, params, expected_output):
     num_workers = [None, 1, 2, 4, 5, 100]
     for n in num_workers:
-        output = audeer.run_worker_threads(func, params, num_workers=n)
+        with pytest.warns(UserWarning):
+            output = audeer.run_worker_threads(func, params, num_workers=n)
         assert output == expected_output
 
 
+@pytest.mark.parametrize(
+    'versions, expected_versions',
+    [
+        (
+            ['1.0.0', '3.0.0', '1.10.1', '1.2.0', '1.0.1'],
+            ['1.0.0', '1.0.1', '1.2.0', '1.10.1', '3.0.0'],
+        ),
+        (
+            [
+                '1.0.0-SNAPSHOT',
+                '4.0.0-20200206.095424-2',
+                '1.0.0',
+                '2.0.0-20200131.102442-1',
+                '3.0.0',
+                '3.1.0',
+                '4.0.0-20200206.095316-1',
+                '3.2.0',
+                '2.0.0-20200131.102728-2',
+                '3.3.0',
+                '3.4.0',
+                '4.0.0-20200206.095534-3',
+                '4.0.0',
+            ],
+            [
+                '1.0.0',
+                '1.0.0-SNAPSHOT',
+                '2.0.0-20200131.102442-1',
+                '2.0.0-20200131.102728-2',
+                '3.0.0',
+                '3.1.0',
+                '3.2.0',
+                '3.3.0',
+                '3.4.0',
+                '4.0.0',
+                '4.0.0-20200206.095316-1',
+                '4.0.0-20200206.095424-2',
+                '4.0.0-20200206.095534-3',
+            ],
+        ),
+        (
+            ['v1.0.0', 'v1.0.1', 'v1.0.1-1-gdf29c4a'],
+            ['v1.0.0', 'v1.0.1', 'v1.0.1-1-gdf29c4a'],
+        ),
+        # From https://github.com/postmarketOS/pmbootstrap/issues/342
+        (
+            ['22.7.3-r1.3', '22.7.3-r1'],
+            ['22.7.3-r1', '22.7.3-r1.3'],
+        ),
+        (
+            ['1.0.0', '1.1.1+1', '1.2.1', '1.2.0'],
+            ['1.0.0', '1.1.1+1', '1.2.0', '1.2.1'],
+        ),
+    ]
+)
+def test_sort_versions(versions, expected_versions):
+    sorted_versions = audeer.sort_versions(versions)
+    assert sorted_versions == expected_versions
+
+
+@pytest.mark.parametrize(
+    'versions, error_message',
+    [
+        (
+            ['1'],
+            (
+                "All version numbers have to be semantic versions, "
+                "following 'X.Y.Z', "
+                "where X, Y, Z are integers. "
+                "But your version is: '1'."
+            ),
+        ),
+    ]
+)
+def test_sort_versions_errors(versions, error_message):
+    with pytest.raises(ValueError, match=error_message):
+        audeer.sort_versions(versions)
+
+
 @pytest.mark.parametrize(
     'input,expected_output',
     [
